#! /usr/bin/env python
#*******************************************************************************
# ALMA - Atacama Large Millimiter Array
# (c) Associated Universities Inc., 2009 
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# "@(#) $Id: ObsCalSolarFastScan.py 230915 2016-04-15 16:15:41Z ahirota $"

#
# forcing global imports is due to an OSS problem
#
global math
import math

global Control
import Control

global CCL
import CCL.APDMSchedBlock

global ControlExceptionsImpl
import ControlExceptionsImpl

global Observation
import Observation.PointingCalTarget
import Observation.FocusCalTarget
import Observation.ObsCalBase
# FIXME: this should not be needed in 2015.8+
import Observation.MixerDetuning
import Observation.FillPattern
import Observation.FastScanPattern
import Observation.TargetUtility

global TelCalParameters
import TelCalParameters

class ObsCalSolarFastScan(Observation.ObsCalBase.ObsCalBase):

    pointingExcursion = None
    ifLevel = 30.0
    bbLevel = 2.0

    def __init__(self):
        Observation.ObsCalBase.ObsCalBase.__init__(self, singleDish=True)
        # fudge to use OSS
        #Observation.ObsCalBase.ObsCalBase.__init__(self, singleDish=False)
        self._srcPointFocus = None

    def parseOptions(self):
        self.pointingSubscanDuration = self.getExpertParameterOrDefault("pointingSubscanDuration", float, 3.072)
        self.focusSubscanDuration    = self.getExpertParameterOrDefault("focusSubscanDuration", float, 2.0)
        self.atmSubscanDuration      = self.getExpertParameterOrDefault("atmSubscanDuration", float, 4.0)
        self.numFocusPositions       = self.getExpertParameterOrDefault("numFocusPositions", int, 5)
        self.focusOneWay             = self.getExpertParameterOrDefault("focusOneWay", int, 0) > 0
        self.tpIntegrationDuration   = self.getExpertParameterOrDefault("tpIntegrationDuration", float, 0.001)
        # map parameters
        self.mapPatternType          = self.getExpertParameterOrDefault("mapPatternType", str, "CENTERED")
        self.mapPatternSubtype       = self.getExpertParameterOrDefault("mapPatternSubtype", str, "AUTO")
        self.mapWidth                = self.getExpertParameterOrDefault("mapWidth", float, 200.0)
        self.mapHeight               = self.getExpertParameterOrDefault("mapHeight", float, 200.0)
        self.mapSpacing              = self.getExpertParameterOrDefault("mapSpacing", float, 5.0)
        self.mapOrientation          = self.getExpertParameterOrDefault("mapOrientation", float, 0.0)
        self.mapMaxFreq              = self.getExpertParameterOrDefault("mapMaxFreq", float, 0.5)
	# As we sometimes observe the Moon or Sun we are best to default to a long way away from the source!
        self.atmAzOffsetArcsec       = self.getExpertParameterOrDefault("atmAzOffsetArcsec", float, 3600.0)
        self.atmDoZero               = self.getExpertParameterOrDefault("atmDoZero", int, 1) > 0
        self.elLimit                 = self.getExpertParameterOrDefault("ElLimit", str, "20 deg")
        self.pointFocusBand          = self.getExpertParameterOrDefault("band", int, 3)
        self.sourceName              = self.getExpertParameterOrDefault("source", str, "Sun")
        self.activeSun               = self.getExpertParameterOrDefault("activeSun", int, 0) > 0
        self.atmAzOffsetRad = math.radians(self.atmAzOffsetArcsec / 3600.0)

    def generateTunings(self):
        # point/focus tuning will typically have longer sample time than fast scan map
        self._pointFocusSpectralSpec = self._tuningHelper.GenerateSpectralSpec(
                band = self.pointFocusBand,
                intent = "total_power",
                tpSampleTime = 0.016)
        self._pointFocusSpectralSpec.name = "Band %d pointing/focus" % self.pointFocusBand
        self._mapSpectralSpec = self._tuningHelper.GenerateSpectralSpec(
                band = self.pointFocusBand,
                intent = "total_power",
                tpSampleTime = self.tpIntegrationDuration)
        self._mapSpectralSpec.name = "Band %d mapping" % self.pointFocusBand


    def setMapSource(self):
        self.mapSrc = self.sourceHelper.getSource(self.sourceName)

    def setTelCalParams(self):
        self.logInfo("Setting TelCal parameters pointingFitWidth=False, simpleGaussianFit=False")
	tcParameters = self.getTelCalParams()
	tcParameters.setCalibParameter('pointingFitWidth', False)
        tcParameters.setCalibParameter('simpleGaussianFit',False)

    def configureScanPattern(self):
	frame = Control.HORIZON
	fp = Observation.FillPattern.FillPattern('%.1f arcsec' % self.mapWidth,
			'%.1f arcsec' % self.mapHeight,
			'%.1f arcsec' % self.mapSpacing,
			'%.3f deg' % self.mapOrientation,
			Observation.FillPattern.FillPatternType.getType(self.mapPatternType),
			Observation.FillPattern.FillPatternSubtype.getSubtype(self.mapPatternSubtype))
	fsp = Observation.FastScanPattern.FastScanPattern()
	fsp.limitFrequencyMax = self.mapMaxFreq
	spec = fsp.generatePatternSpec(fp)
	print spec._PatternSpec__idlSpec
	self.mapOffsetSpec = Control.SourceOffset(longitudeOffset=0.0, latitudeOffset=0.0,
                                  longitudeVelocity=0.0, latitudeVelocity=0.0,
                                  pattern = spec._PatternSpec__idlSpec,
                                  frame=frame);


    # This should probably move to PointingCalTarget
    def setPointingExcursion(self):
	clight = 299792458.0
	antennaDiameter = 12.0
        # doing this here where we can't account for pointing subarrays is a bit dumb
	for ant in self._array.antennas():
		if ant[0] == 'C':
			antennaDiameter = 7.0
			self.logInfo("Array contains at least one CM antenna, so using D=7m for beam sizes.")
			break
        obsFreq = self._pointFocusSpectralSpec.getMeanFrequency()
        sourceDiameter = 0.0
        try:
            sourceDiameter = Observation.TargetUtility.getAngularDiameter(self._srcPointFocus)
            if sourceDiameter is None: sourceDiameter = 0.0
        except: pass
	self.logInfo("Assumed source diameter: %.2f arcsec" % (3600.0*math.degrees(sourceDiameter)))
	beam = 1.22*(clight/obsFreq)/antennaDiameter
        excursion = math.sqrt(4.0*beam*beam + sourceDiameter*sourceDiameter)
	self.logInfo("Assumed antenna diamter: %.1f m" % (antennaDiameter))
	self.logInfo("Assumed beam FWHM:       %.2f arcsec" % (beam * (3600.0 * 180.0 / math.pi)))
        self.logInfo("Chosen excursion:        %.2f arcsec" % (excursion * (3600.0 * 180.0 / math.pi)))
        self.pointingExcursion = excursion


    def doPointing(self):
        if self.pointingExcursion is None:
            self.setPointingExcursion()
        try:
            pointingCal = Observation.PointingCalTarget.PointingCalTarget(self._srcPointFocus, self._pointFocusSpectralSpec)
            pointingCal.setSubscanDuration(self.pointingSubscanDuration)
            pointingCal.setDataOrigin('TOTAL_POWER')
            pointingCal.setDelayCalReduction(False)
            pointingCal.setPointingMethod('cross')
            pointingCal.setExcursion(self.pointingExcursion)
            pointingCal.setWVRCalReduction(False)
            self.logInfo('Executing PointingCal on ' + self._srcPointFocus.sourceName + '...')
            pointingCal.execute(self._obsmode)
            self.logInfo('Completed PointingCal on ' + self._srcPointFocus.sourceName)
            result = pointingCal.checkResult(self._array)
            self.logInfo("Result is: %s" % str(result))
            if len(result) > 0: 
                for key in result.keys():
                    self.logInfo("Found solution for %s using polarization(s) %s" %
                            (key, result[key]))
                pointingCal.applyResult(self._obsmode, result)
            else:
                if not "OSS" in self._array._arrayName:
                    raise Exception("No pointing results!")
        except BaseException, ex:
            print ex
            msg = "Error executing pointing on source %s" % self._srcPointFocus.sourceName
            self.logError(msg)
            self.closeExecution(ex)
            raise ex


    def doFocus(self):
        ref = CCL.APDMSchedBlock.Reference()
        ref.setCoordinates('-600 arcsec',0,u'horizon')
        ref.cycleTime.set(1.0)
        ref.subScanDuration.set(self.focusSubscanDuration)
        ref.integrationTime.set(self.focusSubscanDuration)
        self._srcPointFocus.Reference = [ref]
        try:
            focusCal = Observation.FocusCalTarget.FocusCalTarget(self._srcPointFocus, self._pointFocusSpectralSpec)
            focusCal.setSubscanDuration(self.focusSubscanDuration)
            focusCal.setIntegrationTime(3.0 * self.focusSubscanDuration)
            focusCal.setDataOrigin('TOTAL_POWER')
            focusCal.setNumPositions(self.numFocusPositions)
            focusCal.setOneWayScan(self.focusOneWay)
            focusCal.setWVRCalReduction(False)
            self.logInfo('Executing FocusCal on ' + self._srcPointFocus.sourceName + '...')
            focusCal.execute(self._obsmode)
            self.logInfo('Completed FocusCal on ' + self._srcPointFocus.sourceName)
            result = focusCal.checkResult(self._array)
            self.logInfo("Result is: %s" % str(result))
            if len(result) > 0: 
                for key in result.keys():
                    self.logInfo("Found solution for %s using polarization(s) %s" %
                             (key, result[key]))
                focusCal.applyResult(self._obsmode, result)
            else:
                if not "OSS" in self._array._arrayName:
                    raise Exception("No focus results!")
        except BaseException, ex:
            print ex
            msg = "Error executing focus on source %s" % self._srcPointFocus.sourceName
            self.logError(msg)
            self.closeExecution(ex)
            raise ex


    def doAtmCal(self):
        try:
                atm = Observation.AtmCalTarget.AtmCalTarget(self.mapSrc, self._mapSpectralSpec, doHotLoad=True)
                atm.setOnlineProcessing(True)
                atm.setDoZero(self.atmDoZero)
                atm.setSubscanDuration(self.atmSubscanDuration)
                atm.setIntegrationTime(1.5)
		if self._mapSpectralSpec.hasCorrelatorConfiguration():
			atm.setDataOrigin('FULL_RESOLUTION_AUTO')
			atm.setWVRCalReduction(True)
		else:
			atm.setDataOrigin('TOTAL_POWER')
			atm.setWVRCalReduction(False)
                atm.setApplyWVR(False)
                # Currently we need to set a reference source to use an offset.
                atm._referenceSource=atm._source
                atm._referenceOffset=CCL.SourceOffset.stroke(self.atmAzOffsetRad,0,0,0,Control.HORIZON)
                self.logInfo('Executing AtmCal on ' + self.mapSrc.sourceName + '...')
                atm.execute(self._obsmode)
                self.logInfo('Completed AtmCal on ' + self.mapSrc.sourceName)
        except BaseException, ex:
                print ex
                msg = "Error executing AtmCal on source %s" % self.mapSrc.sourceName
                self.logError(msg)
                self.closeExecution(ex)
                raise ex

    def doMap(self):
		ssl = CCL.SubscanList.SubscanList()
		attenuatorSettingName = "Map attenuator setting"
                subscanDuration = 0.048*(1 + int(options.subscanDuration / 0.048))
		ssl.addSubscan(self.mapSrc, self._mapSpectralSpec, subscanDuration, SubscanIntent=PySubscanIntent.ON_SOURCE, CalibrationDevice=calDevice,
				PointingOffsetSpec = self.mapOffsetSpec,
		#		attenuatorSettingName = "Solar Attenuator Setting")
		#		optimizeAttenuators=[levels.getOptimumIFLevel(spectralSpec), levels.getOptimumBBLevel(spectralSpec)],
				attenuatorSettingName = attenuatorSettingName)
		if options.doRef:
			referenceOffset=CCL.SourceOffset.stroke(math.radians(self.atmAzOffsetRad,0,0,0,Control.HORIZON))
			self.logInfo("referenceOffset = %s" % str(referenceOffset))
			ssl.addSubscan(src, spectralSpec, options.refSubscanDuration, SubscanIntent=PySubscanIntent.OFF_SOURCE, CalibrationDevice=calDevice,
					PointingOffsetSpec = referenceOffset,
					attenuatorSettingName = attenuatorSettingName)
		subscanSpec = ssl.getSubscanSequenceSpecification()
                self.logInfo('subscanSpec.subscans = %s' % str(subscanSpec.subscans))
		scanIntent = CCL.ScanIntent.ScanIntent(PyScanIntent.OBSERVE_TARGET)
		self.logInfo("Now executing the scan...")
		obsmode.beginScan(scanIntent)

		# Just before execution turn off the focus model when pointed at the source
		obsmode.setPointingDirection(self.mapSrc)
		#for ant in self._array.antennas():
		#	CCL.Mount.Mount(ant).enableFocusModel(False)

		# the obvserving mode seems to set the levels after statting the motion pattern, so we have to do it ourselves and save the result. Ugh.
		self.logInfo("Optimising attenuators and saving to setting named '%s'" % attenuatorSettingName)
                loMode = self._obsmode.getLOObservingMode()
                loMode.optimizeSignalLevels(ifTargetLevel = self.ifLevel, bbTargetLevel = self.bbLevel, settingName = attenuatorSettingName)

		# Sort out correlator calibration IDs
		if subscanSpec.spectra[0].hasCorrelatorConfiguration():
			self.logInfo('Performing correlator calibration via getCalibrationIds() with validity interval %d seconds'
					% obsmode.getCorrelatorCalibrationValidityInterval())
			try:
				obsmode.getCalibrationIds(subscanSpec)
			except Exception, ex:
				self.logException('Exception thrown by obsmode.getCalibrationIds(), considering this fatal!', ex)
		else:
			self.logInfo('No correlator configuration, so just setting calibration IDs to -1 to save effort calling getCalibrationIds()')
			for s in subscanSpec.subscans:
				s.calibrationId = -1

		self.logInfo("Now executing the subscan sequence...")
		obsmode.doSubscanSequence(subscanSpec)
		self.logInfo("Subscan sequence ended")
		obsmode.endScan()
		self.logInfo("Scan ended")

		self.logInfo("Enabling focus models...")
		#for ant in self._array.antennas():
		#	CCL.Mount.Mount(ant).enableFocusModel(False)

		self.logInfo("Done!")


Observation.ObsCalBase.checkArray()

obs = ObsCalSolarFastScan()
obs.parseOptions()
#obs.checkAntennas()
obs.startPrepareForExecution()
obs.generateTunings()
obs.setMapSource()
obs.findPointFocusSource(maxEl=65.0, singleDish=True, sunMoon=obs.sourceName)
obs.configureScanPattern()
obs.completePrepareForExecution()
obs.setTelCalParams()
obs.logInfo("Executing first pointing...")
obs.doPointing()
obs.logInfo("Executing second pointing -- make sure results are good!...")
obs.doPointing()
obs.logInfo("Executing focus...")
obs.doFocus()
obs.logInfo("Executing last pointing -- make sure results are good!...")
obs.doPointing()
obs.logInfo("Executing AtmCal...")
obs.doAtmCal()
obs.logInfo("Executing Map...")
obs.doMap()
obs.closeExecution()

